'From Squeak5.1 of 23 July 2016 [latest update: #16138] on 28 July 2016 at 2:37:33 pm'!
Object subclass: #UserInterfaceTheme
	instanceVariableNames: 'scope properties name next'
	classVariableNames: 'All Current Default'
	poolDictionaries: ''
	category: 'System-Support'!

!UserInterfaceTheme commentStamp: '<historical>' prior: 0!
A UserInterfaceTheme is a dictionary of preferred visual-properties; colors, borderStyles, borderWidths, fonts, forms, etc. used to color and style the IDE.

Accessing The Theme
To access the proper UserInterfaceTheme instance for an object, send it #userInterfaceTheme.  The default implementation on Object provides the one instance of that is in-use by the IDE at the current time.

Customizing The Theme
We can ask the userInterfaceTheme for the value of any visual-property, by name:

	mySystemWindow userInterfaceTheme closeBoxImage

Initially, the above answers nil, which causes the legacy code to use whatever default it's always used.  To override various visual-properties of any kind of object, the #set: onAny: to: message can be used.  For example, 

	myUserInterfaceTheme
		set: #closeBoxImage 
		for: SystemWindow
		to: MenuIcons smallCancelIcon

Alternatively, values may be derived based on other values in the theme, as in:

	myUserInterfaceTheme
		set: #color 
		for: FillInTheBlankMorph
		to: { MenuMorph->#color.  #twiceDarker }

Now, the accessing expression, above, will answer will answer MenuIcons' smallCancelIcon instead of nil.  SystemWindow's code can be changed to use the expression above to access elements of the theme.

Upgrading Legacy Code
Following the introduction of this class, various client code all around the system must be modified to access it.  This variety of legacy code uses a variety of methods to specify their visual properties:

	1) a hard-coded values.
	2) a values derived from some other value.
	3) providing local storage for a settable value which can be nil.
	4) providing local storage for a settable value which is expected to always have a particular valid value (never nil).

The requirement, for each case, is to let the value be overridden.  

The solution for each of the above should be handled respectively to the above list, as follows:

	1) Check the userInterfaceTheme, if that property returns nil, use the legacy hard-coded value.  (see example: SystemWindow>>#createCloseBox).
	2) Nothing to do -- simply perform the same derivation on the result of (1).
	3) Check the local storage, if present, use it.  If nil, then check the userInterfaceTheme, if it has this property present, use it, else return nil.
	4) Check the userInterfaceTheme, if the property is not nil, use it, otherwise use the local value.

Tool Support
If a new access to #userInterfaceTheme is added to the code, be sure to add the property and its description to the #themeSettings for that class.  See implementors of #themeSettings for examples.!

TestCase subclass: #UserInterfaceThemeTest
	instanceVariableNames: 'theme previous'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Tests-System-Preferences'!
Object subclass: #UserInterfaceThemeTestObject
	instanceVariableNames: 'testColor unknownProperty'
	classVariableNames: 'ApplyCounter'
	poolDictionaries: ''
	category: 'Tests-System-Preferences'!

!Object methodsFor: '*System-Preferences' stamp: 'mt 7/1/2016 17:50'!
applyUserInterfaceTheme
	"A new UserInterfaceTheme has been asked to #apply itself onto the current desktop.  The receiver should do what is necessary to change its colors on the screen, update its fonts, form images, etc., if the current UserInterfaceTheme specifies overrides for those values.
	
	Only subclasses which implement new #themeProperties on the class-side should implement this method.
	Implementors of this method should not call 'super applyUserInterfaceTheme'.

	The default implementation does nothing."! !

!Object methodsFor: '*System-Preferences' stamp: 'mt 7/1/2016 17:50'!
canApplyUserInterfaceTheme
	"Use this hook to control whether you want to get notified if a new theme gets applied."

	^ true! !

!Object methodsFor: '*System-Preferences' stamp: 'mt 7/1/2016 17:51'!
userInterfaceTheme
	"Call this to conveniently access properties from the current user interface theme."

	^ UserInterfaceTheme current
		pushScope: self;
		yourself! !


!ClassDescription methodsFor: '*System-Support' stamp: 'mt 7/1/2016 17:10'!
applyThemeToInstances
	"User interface themes. Manage how to update existing instances. Classes can override this to use a different update schema than iterating over all instances."
	
	self allInstancesDo:
		[ : eachInstance | eachInstance canApplyUserInterfaceTheme ifTrue: [ eachInstance applyUserInterfaceTheme ] ]! !

!ClassDescription methodsFor: '*System-Support' stamp: 'mt 7/3/2016 14:35'!
canApplyThemeToInstances
	"Override this to ignore your instances."
	
	^ true! !


!Morph methodsFor: 'visual properties' stamp: 'cmm 5/30/2016 19:47'!
canApplyUserInterfaceTheme
	^ self isInWorld! !


!Object class methodsFor: '*System-Preferences' stamp: 'mt 7/1/2016 17:44'!
themeProperties
	"Return a list of triples that describe configurable properties in a theme."
	^ {}! !


!UserInterfaceTheme methodsFor: 'building' stamp: 'mt 7/25/2016 14:59'!
clear: propertySymbol
	
	^ self
		set: propertySymbol
		to: nil! !

!UserInterfaceTheme methodsFor: 'building' stamp: 'cmm 5/23/2016 14:11'!
clear: propertySymbol for: aClass
	"No longer override the default visual property specified by propertySymbol for any kinds of aClass.  Use the default."
	^ self
		set: propertySymbol
		for: aClass
		to: nil! !

!UserInterfaceTheme methodsFor: 'building' stamp: 'mt 7/3/2016 13:08'!
derive: propertySymbol for: aClassOrSymbol from: anotherClassOrSymbol

	^ self
		derive: propertySymbol
		for: aClassOrSymbol
		from: anotherClassOrSymbol
		at: propertySymbol
! !

!UserInterfaceTheme methodsFor: 'building' stamp: 'mt 7/3/2016 13:09'!
derive: propertySymbol for: aClassOrSymbol from: anotherClassOrSymbol at: anotherPropertySymbol
	"Derive this property from another property."
	
	self
		set: propertySymbol
		for: aClassOrSymbol
		to: (MessageSend
			receiver: self
			selector: #get:for:
			arguments: {anotherPropertySymbol. anotherClassOrSymbol}).! !

!UserInterfaceTheme methodsFor: 'building' stamp: 'mt 7/3/2016 13:08'!
derive: propertySymbol for: aClassOrSymbol from: anotherClassOrSymbol at: anotherPropertySymbol do: block
	
	self
		set: propertySymbol
		for: aClassOrSymbol
		to: (MessageSend
			receiver: self
			selector: #get:for:do:
			arguments: {anotherPropertySymbol. anotherClassOrSymbol. block}).! !

!UserInterfaceTheme methodsFor: 'building' stamp: 'mt 7/3/2016 13:09'!
derive: propertySymbol for: aClassOrSymbol from: anotherClassOrSymbol do: block

	^ self
		derive: propertySymbol
		for: aClassOrSymbol
		from: anotherClassOrSymbol
		at: propertySymbol
		do: block! !

!UserInterfaceTheme methodsFor: 'building' stamp: 'cmm 5/25/2016 18:15'!
link: aUserInterfaceTheme
	"When accessing properties by name, if a property is not specified in the receiver, give aUserInterfaceTheme a chance to provide it."
	next := aUserInterfaceTheme! !

!UserInterfaceTheme methodsFor: 'building' stamp: 'cmm 5/25/2016 18:16'!
merge: aUserInterfaceTheme 
	"Merge aUserInterfaceTheme into my properties.  Same as #link:, except merges all the properties of aUserInterfaceTheme into the  receiver"
	self
		merge: aUserInterfaceTheme
		overwrite: false! !

!UserInterfaceTheme methodsFor: 'building' stamp: 'mt 7/3/2016 13:22'!
merge: aUserInterfaceTheme overwrite: aBoolean 
	"Merge aUserInterfaceTheme into my properties. Only overwrite properties if it sais so."
	
	self
		 validateCanMerge: aUserInterfaceTheme ;
		 preMerge: aUserInterfaceTheme.

	self atomicUpdate: [:props |
		aUserInterfaceTheme properties keysAndValuesDo: [:key :value |
			(aBoolean or: [(props includesKey: key) not])
				ifTrue: [ | mergedValue |
					mergedValue := value copy.
					"Update receiver when deriving properties."
					mergedValue isMessageSend ifTrue: [mergedValue receiver: self].
					props at: key put: mergedValue.
					]]].! !

!UserInterfaceTheme methodsFor: 'building' stamp: 'mt 7/2/2016 12:25'!
set: propertySymbol for: aClassOrSymbol to: valueObject
	"Where aClass asks its userInterfaceTheme for propertySymbol, provide valueObject."
	
	| aClass |
	aClass := aClassOrSymbol isBehavior ifTrue: [aClassOrSymbol] ifFalse: [Smalltalk classNamed: aClassOrSymbol].
	aClass ifNil: [^ self].
	^ self atomicUpdate:
		[ : props | | key |
		key := aClass -> propertySymbol.
		valueObject
			ifNil:
				[ props
					removeKey: key
					ifAbsent: [ "already cleared, don't error" ] ]
			ifNotNil:
				[ props
					at: key
					put: valueObject ] ]! !

!UserInterfaceTheme methodsFor: 'building' stamp: 'mt 7/25/2016 14:54'!
set: propertySymbol to: valueObject
	"For convenience."
	
	^ self atomicUpdate:
		[ : props |
		valueObject
			ifNil:
				[ props
					removeKey: propertySymbol
					ifAbsent: [ "already cleared, don't error" ] ]
			ifNotNil:
				[ props
					at: propertySymbol
					put: valueObject ] ]! !

!UserInterfaceTheme methodsFor: 'initialization' stamp: 'cmm 5/23/2016 14:16'!
initialize
	super initialize.
	name := 'unnamed'.
	properties := Dictionary new.
	scope := Stack new! !

!UserInterfaceTheme methodsFor: 'initialization' stamp: 'mt 7/1/2016 14:49'!
register
	self class allThemes add: self.! !

!UserInterfaceTheme methodsFor: 'initialization' stamp: 'mt 7/1/2016 14:50'!
unregister
	self class allThemes remove: self ifAbsent: [].! !

!UserInterfaceTheme methodsFor: 'accessing' stamp: 'cmm 5/23/2016 14:15'!
name
	^ name! !

!UserInterfaceTheme methodsFor: 'accessing' stamp: 'cmm 5/23/2016 14:15'!
name: aString 
	name := aString! !

!UserInterfaceTheme methodsFor: 'accessing' stamp: 'cmm 5/25/2016 16:24'!
next
	"The next theme to look for properties when I don't contain a particular property."
	^ next! !

!UserInterfaceTheme methodsFor: 'accessing' stamp: 'cmm 6/1/2016 21:31'!
unsetProperties
	"Answer the property keys which are unset."! !

!UserInterfaceTheme methodsFor: 'printing' stamp: 'cmm 5/23/2016 14:17'!
printOn: aStream
	super printOn: aStream.
	aStream space; print: name! !

!UserInterfaceTheme methodsFor: 'private' stamp: 'cmm 5/13/2016 19:26'!
atomicUpdate: aBlock
	"Like our Preferences >> #atomicUpdatePreference"
	[
		| original copy returnValue |
		original := properties.
		copy := properties copy.
		returnValue := aBlock value: copy.
		original == properties ifTrue: [
			properties := copy.
			^ returnValue]
	] repeat! !

!UserInterfaceTheme methodsFor: 'private' stamp: 'mt 7/25/2016 18:02'!
get: keyObject 
	"keyObject is intended to be an Association. We have two lookup strategies: 1) along the superclass chain, 2) via a linke theme. Evaluate the result because there can be message sends stored or blocks."
	
	| k |
	properties
		at: keyObject
		ifPresent: [:prop | ^ prop value].
	
	keyObject isVariableBinding "simple key objects"
		ifFalse: [^ self getViaLink: keyObject].
	
	k := keyObject key.
	(self getViaSuperclasses: keyObject)
		ifNotNil: [:prop | ^ prop].
		
	keyObject key: k. "restore"
	^ self getViaLink: keyObject! !

!UserInterfaceTheme methodsFor: 'private' stamp: 'mt 7/9/2016 11:29'!
get: propertySymbol for: scope
	"For convenience. Does support access to non-class keys."
	
	| aClass |
	aClass := (scope isNil or: [scope isBehavior])
		ifTrue: [scope]
		ifFalse: [Smalltalk classNamed: scope].

	aClass ifNotNil: [^ self get: aClass -> propertySymbol].
		
	properties
		at: scope -> propertySymbol
		ifPresent: [:prop | ^ prop value].
		
	^ self getViaLink: scope -> propertySymbol! !

!UserInterfaceTheme methodsFor: 'private' stamp: 'mt 7/3/2016 13:04'!
get: propertySymbol for: scope do: block

	^ block cull: (self get: propertySymbol for: scope)! !

!UserInterfaceTheme methodsFor: 'private' stamp: 'mt 7/3/2016 13:49'!
getViaLink: keyObject 
	"keyObject is intended to be an Association"
	
	^ next ifNotNil: [next get: keyObject]! !

!UserInterfaceTheme methodsFor: 'private' stamp: 'mt 7/25/2016 17:56'!
getViaSuperclasses: keyObject 
	"keyObject is intended to be an Association"
		
	"We know we're the only referencer of keyObject.  Update it rather than create new ones, for performance reasons."
	keyObject key: keyObject key superclass.

	keyObject key ifNil: [^ nil].
	
	properties
		at: keyObject
		ifPresent: [:prop | ^ prop value].
	
	^ self getViaSuperclasses: keyObject! !

!UserInterfaceTheme methodsFor: 'private' stamp: 'cmm 5/25/2016 17:57'!
preMerge: aUserInterfaceTheme 
	name := name , ' + ' , aUserInterfaceTheme name.
	next ifNil: [ next := aUserInterfaceTheme next ]! !

!UserInterfaceTheme methodsFor: 'private' stamp: 'cmm 5/25/2016 15:01'!
properties
	^ properties! !

!UserInterfaceTheme methodsFor: 'private' stamp: 'cmm 5/13/2016 22:07'!
pushScope: anObject
	scope push: anObject! !

!UserInterfaceTheme methodsFor: 'private' stamp: 'cmm 5/25/2016 17:59'!
validateCanMerge: aUserInterfaceTheme 
	(next notNil and:
		[ aUserInterfaceTheme next notNil and: [ next ~= aUserInterfaceTheme next ] ]) ifTrue: [ self error: 'Links to different themes.  Cannot merge.' ]! !

!UserInterfaceTheme methodsFor: 'lookup' stamp: 'mt 7/3/2016 13:48'!
doesNotUnderstand: aMessage 
	"Answer whether I have, or inherit, a value for the visual-attribute specified by aMessage's #selector."

	aMessage numArgs > 0 ifTrue: [^ super doesNotUnderstand: aMessage].
	scope isEmpty ifTrue: [^ super doesNotUnderstand: aMessage].
	
	^ [self get: scope top class -> aMessage selector]
		ensure: [scope pop]! !

!UserInterfaceTheme methodsFor: 'copying' stamp: 'cmm 5/26/2016 15:14'!
postCopy
	"Keep same name and linked next."
	super postCopy.
	properties := properties copy.
	scope := nil! !

!UserInterfaceTheme methodsFor: 'actions' stamp: 'mt 7/3/2016 15:11'!
apply
	"Apply this theme to all affected objects. Let classes decide on how to iterate and call their instances."

	UserInterfaceTheme current: self.

	self class clientClassesToReapply
		do: [:eachClass | eachClass applyUserInterfaceTheme].
	self class clientClassesToReapply
		select: [:eachClass | eachClass canApplyThemeToInstances]
		thenDo: [:eachClass | eachClass applyThemeToInstances].

	Project current restoreDisplay.! !


!UserInterfaceTheme class methodsFor: 'private' stamp: 'mt 7/1/2016 17:01'!
clientClasses

	^ Set new in: [:result |
		self allThemePropertiesDo: [:cls :prop | result add: cls].
		result]! !

!UserInterfaceTheme class methodsFor: 'private' stamp: 'mt 7/1/2016 17:21'!
clientClassesToReapply
	"All client classes plus their unique subclasses."
	
	^ Set new in: [:result | self clientClasses do: [:cc | cc withAllSubclassesDo: [:sc |
		result add: sc]]. result]
! !

!UserInterfaceTheme class methodsFor: 'tools' stamp: 'mt 7/1/2016 14:25'!
allThemeProperties
	"Answer an Array of 3-element Array's.  Each inner Array are the information needed to present a Theme editor tool; the property name, category, and description.
	
	self allThemeProperties"
	
	^ Array streamContents:
		[ : stream | self allThemePropertiesDo: [ : cls : prop | stream nextPut: {cls}, prop ]]! !

!UserInterfaceTheme class methodsFor: 'tools' stamp: 'mt 7/1/2016 14:23'!
allThemePropertiesDo: block
	"Iterate over all classes that provide theme properties."
	
	Smalltalk allClassesDo: [:eachClass |
		(eachClass respondsTo: #themeProperties) ifTrue: [
			eachClass themeProperties do: [:eachProperty |
				block cull: eachClass cull: eachProperty]]].! !

!UserInterfaceTheme class methodsFor: 'tools' stamp: 'mt 7/1/2016 14:25'!
categories
	"self categories"
	
	^ Set new in: [:result |
		self allThemePropertiesDo: [:cls :prop | result add: prop second].
		result]! !

!UserInterfaceTheme class methodsFor: 'tools' stamp: 'cmm 5/30/2016 19:00'!
propertiesForCategory: categoryName
	^ self allThemeProperties select: [ : each | each third = categoryName ]! !

!UserInterfaceTheme class methodsFor: 'tools' stamp: 'mt 7/1/2016 14:26'!
propertiesForClass: aClass
	^ self allThemeProperties select: [ : each | each first = aClass ]! !

!UserInterfaceTheme class methodsFor: 'accessing' stamp: 'mt 7/1/2016 14:49'!
allThemes
	^ All ifNil: [All := IdentitySet new]! !

!UserInterfaceTheme class methodsFor: 'accessing' stamp: 'mt 7/1/2016 14:08'!
current

	^ Current ifNil: [Current := Default]! !

!UserInterfaceTheme class methodsFor: 'accessing' stamp: 'cmm 5/14/2016 22:00'!
current: aUserInterfaceTheme
	"Replace the current system theme with aUserInterfaceTheme."
	Current := aUserInterfaceTheme
"Notify?"! !

!UserInterfaceTheme class methodsFor: 'accessing' stamp: 'mt 7/2/2016 13:56'!
default
	^ Default ifNil: [Default := self new name: 'Autogenerated Default'; yourself]! !

!UserInterfaceTheme class methodsFor: 'accessing' stamp: 'mt 7/1/2016 14:08'!
default: aUserInterfaceTheme
	Default := aUserInterfaceTheme.! !

!UserInterfaceTheme class methodsFor: 'initialize-release' stamp: 'mt 7/1/2016 14:11'!
initialize
	
	self reset.! !

!UserInterfaceTheme class methodsFor: 'initialize-release' stamp: 'mt 7/1/2016 14:55'!
named: aString
	^ self allThemes
		detect: [:ea | ea name = aString]
		ifNone: [self new name: aString; register]! !

!UserInterfaceTheme class methodsFor: 'initialize-release' stamp: 'mt 7/1/2016 14:09'!
reset
	self default apply.! !


!UserInterfaceThemeTest methodsFor: 'tests' stamp: 'mt 7/1/2016 14:35'!
test01ImplementationHooks
	"Any class which implements themeProperties must implement #applyUserInterfaceTheme on the instance side."
	
	| problematicClasses |
	problematicClasses := OrderedCollection new.
	Smalltalk allClassesDo: [ : each | ((each theMetaClass includesSelector: #themeProperties) not
		or: [each theNonMetaClass includesSelector: #applyUserInterfaceTheme])
			ifFalse: [problematicClasses add: each]].
	self assert: problematicClasses isEmpty description: ('UI Theme Violations: {1}' format: {(problematicClasses collect: [:ea | ea name]) joinSeparatedBy: ' '})! !

!UserInterfaceThemeTest methodsFor: 'tests' stamp: 'mt 7/3/2016 13:28'!
test02SetProperty

	| m |
	m := UserInterfaceThemeTestObject new.

	theme set: #testColor for: UserInterfaceThemeTestObject to: Color white.
	self assert: m testColor isNil.

	theme apply.
	self assert: Color white equals: m testColor.
! !

!UserInterfaceThemeTest methodsFor: 'tests' stamp: 'mt 7/1/2016 14:54'!
test03RegisterAndName

	self deny: (UserInterfaceTheme allThemes includes: theme).
	theme register.
	self assert: (UserInterfaceTheme allThemes includes: theme).
	theme unregister.
	self deny: (UserInterfaceTheme allThemes includes: theme).
	
	theme name: 'test03'.
	theme register.
	self assert: theme == (UserInterfaceTheme named: 'test03').
	theme unregister.
	
	self assert: theme ~~ (UserInterfaceTheme named: 'testXX').
	self assert:  (UserInterfaceTheme named: 'testXX') == (UserInterfaceTheme named: 'testXX').
	(UserInterfaceTheme named: 'testXX') unregister.
	self deny: (UserInterfaceTheme allThemes anySatisfy: [:ea | ea name = 'testXX']).! !

!UserInterfaceThemeTest methodsFor: 'tests' stamp: 'mt 7/3/2016 13:28'!
test04SuperClassLookup

	| m |
	m := UserInterfaceThemeTestObject new.

	theme set: #testColor for: Object to: Color white.
	self assert: m testColor isNil.

	theme apply.
	self assert: Color white equals: m testColor.
! !

!UserInterfaceThemeTest methodsFor: 'tests' stamp: 'mt 7/3/2016 13:29'!
test05ClearProperty

	| m |
	m := UserInterfaceThemeTestObject new.
	theme set: #testColor for: UserInterfaceThemeTestObject to: Color white.
	theme apply.

	self assert: Color white equals: m testColor.
	theme clear: #testColor for: UserInterfaceThemeTestObject.
	self assert: Color white equals: m testColor.
	theme apply.
	
	self assert: m testColor isNil.
! !

!UserInterfaceThemeTest methodsFor: 'tests' stamp: 'mt 7/3/2016 13:29'!
test06SetAndClearUnkownProperty
	"Unknown means not defined in #themeProperties und used in code such as #applyUserInterfaceTheme."
	
	| m |
	m := UserInterfaceThemeTestObject new.
	theme set: #unknownProperty for: UserInterfaceThemeTestObject to: #blubb.
	theme apply.

	self assert:#blubb equals: m unknownProperty.
	theme clear: #unknownProperty for: UserInterfaceThemeTestObject.
	self assert:#blubb equals: m unknownProperty.
	theme apply.
	
	self assert: m unknownProperty isNil.
! !

!UserInterfaceThemeTest methodsFor: 'tests' stamp: 'mt 7/1/2016 15:27'!
test07Merge

	| anotherTheme |
	anotherTheme := UserInterfaceTheme new.
	anotherTheme set: #someColor for: Morph to: Color red.
	anotherTheme set: #thirdColor for: Morph to: Color blue.

	theme set: #myColor for: Morph to: Color white.
	theme set: #thirdColor for: Morph to: Color yellow.
	
	self assert: (theme get: #someColor for: Morph) isNil.
	self assert: (anotherTheme get: #myColor for: Morph) isNil.
	
	theme merge: anotherTheme.
	
	self assert: Color red equals: (theme get: #someColor for: Morph).
	self assert: (anotherTheme get: #myColor for: Morph) isNil.

	"No overwrite."
	self assert: Color yellow equals: (theme get: #thirdColor for: Morph).
	self assert: Color blue equals: (anotherTheme get: #thirdColor for: Morph).
! !

!UserInterfaceThemeTest methodsFor: 'tests' stamp: 'mt 7/1/2016 15:28'!
test08MergeAndOverwrite

	| anotherTheme |
	anotherTheme := UserInterfaceTheme new.
	anotherTheme set: #thirdColor for: Morph to: Color blue.
	theme set: #thirdColor for: Morph to: Color yellow.
	
	self assert: Color yellow equals: (theme get: #thirdColor for: Morph).
	self assert: Color blue equals: (anotherTheme get: #thirdColor for: Morph).

	theme merge: anotherTheme overwrite: true.

	self assert: Color blue equals: (theme get: #thirdColor for: Morph).
	self assert: Color blue equals: (anotherTheme get: #thirdColor for: Morph).! !

!UserInterfaceThemeTest methodsFor: 'tests' stamp: 'mt 7/3/2016 13:31'!
test09Link

	| anotherTheme |
	anotherTheme := UserInterfaceTheme new.
	
	anotherTheme set: #testColor for: Object to: Color white.	
	self assert: (theme get: #testColor for: Object) isNil.

	theme link: anotherTheme.
	self assert: theme next == anotherTheme.
	self assert: Color white equals: (theme get: #testColor for: Object).
! !

!UserInterfaceThemeTest methodsFor: 'tests' stamp: 'mt 7/3/2016 13:54'!
test10Blocks

	theme set: #testColor for: Object to: [Color r: 1 g: 1 b: 1].
	self assert: (theme get: #testColor for: Object) ~~ (theme get: #testColor for: Object).
	
! !

!UserInterfaceThemeTest methodsFor: 'tests' stamp: 'mt 7/3/2016 13:56'!
test11LinkAgain

	| anotherTheme yetAnotherTheme |
	anotherTheme := UserInterfaceTheme new.
	yetAnotherTheme := UserInterfaceTheme new.
	
	yetAnotherTheme set: #testColor for: Object to: Color white.
	self assert: (theme get: #testColor for: Object) isNil.

	theme link: anotherTheme.
	anotherTheme link: yetAnotherTheme.
	
	self assert: Color white equals: (theme get: #testColor for: Object).! !

!UserInterfaceThemeTest methodsFor: 'tests' stamp: 'mt 7/1/2016 15:57'!
test12RealDNU
	"If we forgot to push a scope, it is a real DNU and not stack is empty."
	self should: [theme perform: #undefinedMessage] raise: MessageNotUnderstood! !

!UserInterfaceThemeTest methodsFor: 'tests' stamp: 'mt 7/3/2016 13:58'!
test13ClassName
	
	self assert: (theme get: #testColor for: #Object) isNil.

	theme set: #testColor for: #Object to: Color white.
	self assert: Color white equals: (theme get: #testColor for: #Object).
	
	theme clear: #testColor for: #Object.
	self assert: (theme get: #testColor for: #Object) isNil.

	self shouldnt: [theme set: #testColor for: #SomeNonExistentClass to: Color white] raise: Error.
	self assert: (theme get: #testColor for: nil) isNil.! !

!UserInterfaceThemeTest methodsFor: 'tests' stamp: 'mt 7/3/2016 12:07'!
test14Perform
	
	| m |
	m := UserInterfaceThemeTestObject new.
	theme apply.

	self assert: m getTestColor isNil.
	self assert: m getTestColorViaPerform isNil.

	theme set: #testColor for: #UserInterfaceThemeTestObject to: Color white.

	self assert: Color white equals: m getTestColor.
	self assert: Color white equals: m getTestColorViaPerform.! !

!UserInterfaceThemeTest methodsFor: 'tests' stamp: 'mt 7/3/2016 13:08'!
test15DerivedProperties
	
	theme set: #fanciness for: Point to: 42.
	theme derive: #fanciness for: Rectangle from: Point at: #fanciness.

	self assert: 42 equals: (theme get: #fanciness for: Point).
	self assert: 42 equals: (theme get: #fanciness for: Rectangle).! !

!UserInterfaceThemeTest methodsFor: 'tests' stamp: 'mt 7/3/2016 12:55'!
test16ArraysAsProperties
	"Used, for example, by Shout styling. There might be many reasons for storing arrays as properties."
	
	theme set: #complexSpec for: UserInterfaceThemeTestObject to: {#foo. 42. #(a b c)}.
	
	self assert: {#foo. 42. #(a b c)} equals: (theme get: #complexSpec for: UserInterfaceThemeTestObject).! !

!UserInterfaceThemeTest methodsFor: 'tests' stamp: 'mt 7/3/2016 13:13'!
test17DerivedPropertiesWithBlock
	
	theme set: #fanciness for: Point to: 42.
	theme derive: #fanciness for: Rectangle from: Point at: #fanciness do: [:f | f + 1].

	self assert: 42 equals: (theme get: #fanciness for: Point).
	self assert: 43 equals: (theme get: #fanciness for: Rectangle).! !

!UserInterfaceThemeTest methodsFor: 'tests' stamp: 'mt 7/3/2016 13:22'!
test18MergeWithDerivedProperties
	
	| anotherTheme |
	anotherTheme := UserInterfaceTheme new.
	
	theme set: #fanciness for: Point to: 42.
	theme derive: #fanciness for: Rectangle from: Point at: #fanciness do: [:f | f + 1].

	anotherTheme merge: theme.
	theme set: #fanciness for: Point to: 21.

	self assert: 22 equals: (theme get: #fanciness for: Rectangle).
	self assert: 43 equals: (anotherTheme get: #fanciness for: Rectangle).! !

!UserInterfaceThemeTest methodsFor: 'tests' stamp: 'mt 7/3/2016 13:18'!
test19MergeWithCopy
	
	| anotherTheme |
	anotherTheme := UserInterfaceTheme new.
	
	theme set: #someColor for: Object to: Color red.
	anotherTheme merge: theme.

	self assert: (theme get: #someColor for: Object) ~~ (anotherTheme get: #someColor for: Object).! !

!UserInterfaceThemeTest methodsFor: 'tests' stamp: 'mt 7/3/2016 15:14'!
test20ClassSideApply
	
	UserInterfaceThemeTestObject resetApplyCounter.
	self assert: 0 equals: UserInterfaceThemeTestObject applyCounter.
	theme apply.
	self assert: 1 equals: UserInterfaceThemeTestObject applyCounter.
! !

!UserInterfaceThemeTest methodsFor: 'tests' stamp: 'mt 7/9/2016 11:23'!
test21BlocksNoDuplicateEvaluation
	
	| anotherTheme |
	anotherTheme := UserInterfaceTheme new.
	anotherTheme set: #modifier for: Object to: [ [:color | color darker] ].
	
	theme link: anotherTheme.
	self shouldnt: [theme get: #modifier for: Color] raise: Error.	
	self assert: Color yellow darker equals: ((theme get: #modifier for: Color) value: Color yellow).! !

!UserInterfaceThemeTest methodsFor: 'tests' stamp: 'mt 7/25/2016 14:56'!
test22SetAndClearConveniently

	self assert: (theme get: #foo22) isNil.
	theme set: #foo22 to: 22.
	self assert: 22 equals: (theme get: #foo22).
	theme clear: #foo22.
	self assert: (theme get: #foo22) isNil.! !

!UserInterfaceThemeTest methodsFor: 'tests' stamp: 'mt 7/25/2016 17:46'!
test23LookUpReset
	"When nothing is found in the super-class hierarchy, try linked themes. However, start at with the original class again."

	| anotherTheme |
	anotherTheme := UserInterfaceTheme new.
	

	theme set: #fanciness for: Object to: 42.
	theme set: #fanciness for: Point to: 43.
	
	anotherTheme link: theme.

	self assert: 43 equals: (theme get: #fanciness for: Point).
	self assert: 43 equals: (anotherTheme get: #fanciness for: Point).! !

!UserInterfaceThemeTest methodsFor: 'tests' stamp: 'mt 7/25/2016 18:01'!
test24GetSimplePropertiesViaLink

	| anotherTheme |
	anotherTheme := UserInterfaceTheme new.
	
	theme set: #fanciness to: 42.	
	anotherTheme link: theme.

	self assert: 42 equals: (anotherTheme get: #fanciness).! !

!UserInterfaceThemeTest methodsFor: 'tests' stamp: 'mt 7/26/2016 10:07'!
test25DerivedPropertiesViaLink
	"It is not (yet?) possible to reset the look-up for derived properties. You should merge themes if you need it."
	
	| anotherTheme |
	anotherTheme := UserInterfaceTheme new.
	
	theme set: #fanciness for: Point to: 42.
	theme derive: #fanciness for: Rectangle from: Point at: #fanciness do: [:f | f + 1].
	
	anotherTheme set: #fanciness for: Point to: 21.
	anotherTheme link: theme.
	
	self assert: 43 equals: (theme get: #fanciness for: Rectangle).
	"self assert: 22 equals: (anotherTheme get: #fanciness for: Rectangle)."
	self assert: 43 equals: (anotherTheme get: #fanciness for: Rectangle).! !

!UserInterfaceThemeTest methodsFor: 'running' stamp: 'mt 7/1/2016 14:55'!
setUp

	super setUp.
	previous := UserInterfaceTheme current.
	theme := UserInterfaceTheme new name: 'ui theme test'.! !

!UserInterfaceThemeTest methodsFor: 'running' stamp: 'mt 7/3/2016 13:26'!
tearDown

	previous
		ifNil: [UserInterfaceTheme reset]
		ifNotNil: [UserInterfaceTheme current == previous
			ifFalse: [previous apply]].
		
	super tearDown.! !


!UserInterfaceThemeTestObject methodsFor: 'accessing' stamp: 'mt 7/1/2016 14:41'!
testColor

	^ testColor! !

!UserInterfaceThemeTestObject methodsFor: 'accessing' stamp: 'mt 7/1/2016 14:41'!
testColor: anObject

	testColor := anObject! !

!UserInterfaceThemeTestObject methodsFor: 'accessing' stamp: 'mt 7/1/2016 15:03'!
unknownProperty

	^ unknownProperty! !

!UserInterfaceThemeTestObject methodsFor: 'accessing' stamp: 'mt 7/1/2016 15:03'!
unknownProperty: anObject

	unknownProperty := anObject! !

!UserInterfaceThemeTestObject methodsFor: 'updating' stamp: 'mt 7/1/2016 15:39'!
applyUserInterfaceTheme

	self testColor: self userInterfaceTheme testColor.
	self unknownProperty: self userInterfaceTheme unknownProperty.! !

!UserInterfaceThemeTestObject methodsFor: 'updating' stamp: 'mt 7/1/2016 15:49'!
getTestColor

	^ self userInterfaceTheme testColor! !

!UserInterfaceThemeTestObject methodsFor: 'updating' stamp: 'mt 7/3/2016 12:02'!
getTestColorViaPerform

	^ self userInterfaceTheme perform: #testColor! !

!UserInterfaceThemeTestObject methodsFor: 'visual properties' stamp: 'mt 7/1/2016 14:43'!
canApplyUserInterfaceTheme
	^ true! !


!UserInterfaceThemeTestObject class methodsFor: 'benchmark' stamp: 'mt 7/25/2016 17:58'!
benchLookup
	"
	Microsoft Surface Pro 3, Windows 10 v1511, CogVM r201606301459, Squeak 5.1alpha #16138
	   '210,000 per second. 4.76 microseconds per run.' -- leaves enough room for quirky morphs that keep on drawing themselves based on direct theme lookup.
	
	I think this setup is really heavy. Morphs should not look-up things that often. They can cache. Anyway:
	- link through 3 themes
	- look up superclasses up to ProtoObject (for each theme!!)
	
	self benchLookup"
	
	| c t1 t2 t3 m result |
	c := UserInterfaceTheme current.
	m := UserInterfaceThemeTestObject new.
	t1 := UserInterfaceTheme new name: #benchmarkOne.
	t2 := UserInterfaceTheme new name: #benchmarkTwo.
	t3 := UserInterfaceTheme new name: #benchmarkThree.
	
	t3 set: #testColor for: ProtoObject to: Color white.
	t1 link: t2.
	t2 link: t3.
	
	t1 apply.
	
	result := OrderedCollection new.
	
	[
		3 timesRepeat: [result add: [m getTestColor] bench].
	] ensure: [c apply].

	result explore.! !

!UserInterfaceThemeTestObject class methodsFor: 'preferences' stamp: 'mt 7/1/2016 14:38'!
themeProperties

	^ super themeProperties, {
		{#testColor. 'test'. 'Some test property'}
	}! !

!UserInterfaceThemeTestObject class methodsFor: 'as yet unclassified' stamp: 'mt 7/3/2016 15:14'!
applyCounter
	^ ApplyCounter! !

!UserInterfaceThemeTestObject class methodsFor: 'as yet unclassified' stamp: 'mt 7/3/2016 15:21'!
applyUserInterfaceTheme
	ApplyCounter := (ApplyCounter ifNil: [0]) + 1.! !

!UserInterfaceThemeTestObject class methodsFor: 'as yet unclassified' stamp: 'mt 7/3/2016 15:12'!
resetApplyCounter
	ApplyCounter := 0.! !

UserInterfaceTheme initialize!

!UserInterfaceTheme class reorganize!
('private' clientClasses clientClassesToReapply)
('tools' allThemeProperties allThemePropertiesDo: categories fixShadow propertiesForCategory: propertiesForClass:)
('accessing' allThemes current current: default default:)
('initialize-release' initialize named: reset)
!


!Object reorganize!
('accessing' addInstanceVarNamed:withValue: at: at:modify: at:put: basicAddInstanceVarNamed:withValue: basicAt: basicAt:put: basicSize bindWithTemp: enclosedSetElement ifNil:ifNotNilDo: ifNotNilDo: ifNotNilDo:ifNil: in: presenter readFromString: size yourself)
('associating' ->)
('binding' bindingOf:)
('casing' caseOf: caseOf:otherwise:)
('class membership' class inheritsFromAnyIn: isKindOf: isKindOf:orOf: isMemberOf: respondsTo: xxxClass)
('comparing' = closeTo: hash identityHashPrintString literalEqual: ~=)
('converting' adaptToFloat:andCompare: adaptToFloat:andSend: adaptToFraction:andCompare: adaptToFraction:andSend: adaptToInteger:andCompare: adaptToInteger:andSend: adaptToScaledDecimal:andCompare: as: asOrderedCollection asSetElement asString asStringOrText complexContents mustBeBoolean mustBeBooleanIn: printDirectlyToDisplay withoutListWrapper)
('copying' clone copy copyAddedStateFrom: copyFrom: copySameFrom: copyTwoLevel deepCopy initialDeepCopierSize postCopy shallowCopy veryDeepCopy veryDeepCopySibling veryDeepCopyUsing: veryDeepCopyWith: veryDeepFixupWith: veryDeepInner:)
('debugging' haltIf: needsWork)
('debugging-haltOnce' checkHaltCountExpired clearHaltOnce decrementAndCheckHaltCount decrementHaltCount doExpiredHaltCount doExpiredHaltCount: doExpiredInspectCount halt:onCount: haltOnCount: haltOnce haltOnce: haltOnceEnabled hasHaltCount inspectOnCount: inspectOnce inspectUntilCount: removeHaltCount setHaltCountTo: setHaltOnce toggleHaltOnce)
('dependents access' addDependent: breakDependents canDiscardEdits dependents evaluate:wheneverChangeIn: hasUnacceptedEdits myDependents myDependents: release removeDependent:)
('drag and drop' acceptDroppingMorph:event:inMorph: dragPassengerFor:inMorph: dragStartedFor:transferMorph: dragTransferTypeForMorph: wantsDroppedMorph:event:inMorph:)
('error handling' assert: assert:description: assert:descriptionBlock: backwardCompatibilityOnly: caseError deprecated deprecated: deprecated:block: doesNotUnderstand: dpsTrace: dpsTrace:levels: dpsTrace:levels:withContext: error error: halt halt: handles: notify: notify:at: primitiveFailed primitiveFailed: shouldBeImplemented shouldNotImplement subclassResponsibility traitConflict)
('evaluating' value valueWithArguments:)
('filter streaming' byteEncode: drawOnCanvas: elementSeparator encodePostscriptOn: flattenOnStream: fullDrawPostscriptOn: putOn: writeOnFilterStream:)
('flagging' isThisEverCalled isThisEverCalled: logEntry logExecution logExit)
('graph model' addModelYellowButtonMenuItemsTo:forMorph:hand: hasModelYellowButtonMenuItems)
('macpal' codeStrippedOut: contentsChanged currentEvent currentHand currentWorld flash instanceVariableValues isUniversalTiles objectRepresented refusesToAcceptCode scriptPerformer slotInfo)
('message handling' executeMethod: perform: perform:orSendTo: perform:with: perform:with:with: perform:with:with:with: perform:with:with:with:with: perform:with:with:with:with:with: perform:withArguments: perform:withArguments:inSuperclass: perform:withEnoughArguments: with:executeMethod: with:with:executeMethod: with:with:with:executeMethod: with:with:with:with:executeMethod: withArgs:executeMethod:)
('objects from disk' comeFullyUpOnReload: convertToCurrentVersion:refStream: fixUponLoad:seg: objectForDataStream: readDataFrom:size: saveOnFile saveOnFileNamed: storeDataOn:)
('printing' fullPrintString isLiteral longPrintOn: longPrintOn:limitedTo:indent: longPrintString longPrintStringLimitedTo: nominallyUnsent: printOn: printString printStringLimitedTo: printWithClosureAnalysisOn: reportableSize storeOn: storeString stringForReadout stringRepresentation)
('scripting' adaptedToWorld: defaultFloatPrecisionFor: evaluateUnloggedForSelf: methodInterfacesForCategory:inVocabulary:limitClass: methodInterfacesForInstanceVariablesCategoryIn: methodInterfacesForScriptsCategoryIn: selfWrittenAsIll selfWrittenAsIm selfWrittenAsMe selfWrittenAsMy selfWrittenAsThis)
('system primitives' asOop className creationStamp instVarAt: instVarAt:put: instVarNamed: instVarNamed:put: isPinned oopString pin primitiveChangeClassTo: rootStubInImageSegment: setPinned: someObject unpin)
('testing' beViewed belongsToUniClass costumes haltIfNil hasLiteralSuchThat: isArray isBehavior isBlock isCharacter isClosure isCollection isColor isColorForm isCompiledMethod isComplex isContext isDictionary isFloat isForm isFraction isHeap isInteger isInterval isMessageSend isMethodContext isMethodProperties isMorph isMorphicEvent isMorphicModel isNumber isPlayer isPoint isPrimitiveCostume isPseudoContext isRectangle isScriptEditorMorph isSketchMorph isStream isString isSymbol isSystemWindow isText isTextView isTrait isTransparent isVariableBinding isWebBrowser isWindowForModel: knownName name nameForViewer renameInternal: renameTo: shouldBePrintedAsLiteral shouldBePrintedAsLiteralVisiting: showDiffs stepAt:in: stepIn: stepTime stepTimeIn: vocabularyDemanded wantsDiffFeedback wantsSteps wantsStepsIn:)
('updating' changed changed: changed:with: handledListVerification noteSelectionIndex:for: okToChange okToClose update: update:with: updateListsAndCodeIn: windowIsClosing)
('user interface' addModelItemsToWindowMenu: addModelMenuItemsTo:forMorph:hand: asExplorerString defaultLabelForInspector launchPartVia: launchPartVia:label: launchTileToRefer modelSleep modelWakeUp modelWakeUpIn: mouseUpBalk: notYetImplemented windowActiveOnFirstClick windowReqNewLabel:)
('*monticello' isConflict)
('*services-base' requestor)
('*system-support' systemNavigation)
('*Tools-Explorer' explore exploreAndYourself exploreWithLabel:)
('*tools-browser' browseHierarchy)
('private' errorImproperStore errorNonIntegerIndex errorNotIndexable errorSubscriptBounds: species storeAt:inTempFrame:)
('thumbnail' iconOrThumbnailOfSize:)
('*universes')
('*Morphic-Explorer' hasContentsInExplorer)
('futures' future future: futureDo:at:args: futureSend:at:args:)
('*Etoys-viewer' assureUniClass browseOwnClassSubProtocol categoriesForViewer: categoriesForVocabulary:limitClass: chooseNewNameForReference defaultLimitClassForVocabulary: defaultNameStemForInstances elementTypeFor:vocabulary: externalName graphicForViewerTab hasUserDefinedSlots infoFor:inViewer: initialTypeForSlotNamed: isPlayerLike methodInterfacesInPresentationOrderFrom:forCategory: newScriptorAround: newTileMorphRepresentative offerViewerMenuFor:event: offerViewerMenuForEvt:morph: renameScript: tilePhrasesForCategory:inViewer: tilePhrasesForMethodInterfaces:inViewer: tilePhrasesForSelectorList:inViewer: tileToRefer uniqueInstanceVariableNameLike:excluding: uniqueNameForReference uniqueNameForReferenceFrom: uniqueNameForReferenceOrNil updateThresholdForGraphicInViewerTab usableMethodInterfacesIn:)
('*Protocols' currentVocabulary haveFullProtocolBrowsed haveFullProtocolBrowsedShowingSelector:)
('*Etoys-tiles' basicType)
('*Tools-MessageSets' browseAllCallsOn: browseAllImplementorsOf:)
('*Tools-multi-window support' canHaveUnacceptedEdits)
('*morphic' asDraggableMorph asMorph asStringMorph asTextMorph isPluggableListMorph openAsMorph)
('*MorphicExtras-Undo' capturedState commandHistory purgeAllCommands redoFromCapturedState: refineRedoTarget:selector:arguments:in: refineUndoTarget:selector:arguments:in: rememberCommand: rememberUndoableAction:named: undoFromCapturedState:)
('*MorphicExtras-PartsBin' descriptionForPartsBin)
('tracing' inboundPointers inboundPointersExcluding: outboundPointers outboundPointersDo:)
('*Tools-Debugger' canonicalArgumentName chasePointers explorePointers shouldFollowOutboundPointers)
('*System-Object Events-accessing' actionForEvent: actionForEvent:ifAbsent: actionMap actionSequenceForEvent: actionsDo: createActionMap hasActionForEvent: setActionSequence:forEvent: updateableActionMap)
('*System-Change Notification-events' actionsWithReceiver:forEvent: renameActionsWithReceiver:forEvent:toEvent:)
('*System-Change Notification-converting' asActionSequence asActionSequenceTrappingErrors)
('*System-Tools-breakpoint' break)
('*ToolBuilder-Kernel-error handling' confirm: confirm:orCancel:)
('*EToys-user interface' eToyStreamedRepresentationNotifying:)
('*ToolBuilder-Kernel-user interface' inform:)
('*System-Support-user interface' initialExtent)
('*System-Localization-locales' localeChanged localeChangedGently)
('*System-Object Events-accessing-removing' releaseActionMap removeAction:forEvent: removeActionsForEvent: removeActionsSatisfying: removeActionsSatisfying:forEvent: removeActionsWithReceiver: removeActionsWithReceiver:forEvent:)
('*System-Object Events-accessing-triggering' triggerEvent: triggerEvent:ifNotHandled: triggerEvent:with: triggerEvent:with:ifNotHandled: triggerEvent:withArguments: triggerEvent:withArguments:ifNotHandled:)
('*System-Object Events-accessing-registering' when:evaluate: when:send:to: when:send:to:with: when:send:to:withArguments:)
('*Tools-inspecting' basicInspect inspect inspectWithLabel: inspectorClass)
('*Traits' explicitRequirement requirement)
('*Graphics-KernelExtensions' fullScreenSize)
('*System-Finalization' actAsExecutor executor finalizationRegistry finalize hasMultipleExecutors retryWithGC:until: toFinalizeSend:to:with:)
('*collections' compareSafely:)
('*Tools-Browsing' browse)
('*System-Recovery-error handling' primitiveError:)
('*Tools-error handling' notifyWithLabel:)
('*SUnit-testing' isTestClass)
('*51Deprecated' defaultBackgroundColor dragAnimationFor:transferMorph:)
('*Morphic-Events-Filtering' filterEvent:for:)
('*System-Preferences' applyUserInterfaceTheme canApplyUserInterfaceTheme userInterfaceTheme)
!

"Postscript:
Leave the line above, and replace the rest of this comment by a useful one.
Executable statements should follow this comment, and should
be separated by periods, with no exclamation points (!!).
Be sure to put any further comments in double-quotes, like this one."

"Update shadow caches."

!

