'From Squeak5.1 of 23 July 2016 [latest update: #16138] on 28 July 2016 at 2:37:25 pm'!
Morph subclass: #DockingBarMorph
	instanceVariableNames: 'originalColor gradientRamp fillsOwner avoidVisibleBordersAtEdge autoGradient selectedItem activeSubMenu oldKeyboardFocus oldMouseFocus '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Morphic-Menus-DockingBar'!
Morph subclass: #MenuMorph
	instanceVariableNames: 'defaultTarget selectedItem stayUp popUpOwner activeSubMenu '
	classVariableNames: 'CloseBoxImage CloseBoxImageFlat CloseBoxImageGradient GradientMenu MenuBorderColor MenuBorderWidth MenuColor MenuLineColor MenuSelectionColor MenuTitleBorderColor MenuTitleBorderWidth MenuTitleColor PushPinImage RoundedMenuCorners '
	poolDictionaries: ''
	category: 'Morphic-Menus'!

!DockingBarMorph methodsFor: 'accessing' stamp: 'mt 7/26/2016 12:51'!
color: aColor 
	"Set the receiver's color."

	super color: aColor.
	originalColor := aColor asColor.
	gradientRamp := nil.
""
self updateColor! !

!DockingBarMorph methodsFor: 'construction' stamp: 'mt 7/9/2016 13:05'!
addLine
	"Append a divider line to this menu. Suppress duplicate lines."

	| colorToUse |
	submorphs isEmpty ifTrue: [^ self].
	self lastSubmorph knownName = #line ifTrue: [^ self].
	
	colorToUse := self userInterfaceTheme lineColor ifNil: [Color gray: 0.9].
	self addMorphBack: (Morph new
		color: colorToUse;
		extent: (self userInterfaceTheme lineWidth ifNil: [2]) asPoint;
		borderStyle: (self userInterfaceTheme lineStyle ifNil: [BorderStyle inset]);
		borderColor: colorToUse;
		borderWidth: 1;
		name: #line; "see above"
		yourself).! !

!DockingBarMorph methodsFor: 'private - layout' stamp: 'mt 7/26/2016 12:51'!
gradientRamp
	| cc |
	cc := originalColor adjustSaturation: -0.08 brightness: 0.4.
	^ gradientRamp ifNil:[gradientRamp := {
			0.0 -> cc. 
			0.25 -> (originalColor mixed: 0.5 with: cc). 
			0.9 -> originalColor.
			1.0 -> originalColor darker.}]! !

!DockingBarMorph methodsFor: 'private - layout' stamp: 'mt 7/25/2016 14:01'!
updateColor
	"private - update the receiver's color"
	| fill |
	MenuMorph gradientMenu ifFalse: [
		self color ~= originalColor ifTrue: [self color: originalColor].
		^ self].
	""
	fill := GradientFillStyle ramp: self gradientRamp.
	""
	fill origin: self topLeft.
	self isVertical
		ifTrue: [fill direction: self width @ 0]
		ifFalse: [fill direction: 0 @ self height].
	""
	self fillStyle: fill! !

!DockingBarMorph methodsFor: 'private - layout' stamp: 'mt 7/9/2016 13:10'!
updateLayoutProperties
	"private - update the layout properties based on adhering,  
	fillsOwner and avoidVisibleBordersAtEdge preferencs"
	""
	(self isHorizontal
			or: [self isFloating])
		ifTrue: [self listDirection: #leftToRight]
		ifFalse: [self listDirection: #topToBottom].
	""
	self hResizing: #shrinkWrap.
	self vResizing: #shrinkWrap.
	self fillsOwner
		ifTrue: [""
			self isHorizontal
				ifTrue: [self hResizing: #spaceFill].
			self isVertical
				ifTrue: [self vResizing: #spaceFill]].
	""
	self isHorizontal ifTrue: [
		self submorphsDo: [:m | m hResizing: #shrinkWrap; vResizing: #spaceFill]].
	self isVertical ifTrue: [
		self submorphsDo: [:m | m vResizing: #shrinkWrap; hResizing: #spaceFill]].
! !

!DockingBarMorph methodsFor: 'initialize-release' stamp: 'mt 7/9/2016 13:42'!
initialize
	"initialize the receiver"
	super initialize.
	self changeTableLayout.
	
	selectedItem := nil.
	activeSubMenu := nil.
	fillsOwner := true.
	avoidVisibleBordersAtEdge := true.
	autoGradient := MenuMorph gradientMenu.
	self 
		setDefaultParameters ; 
		beFloating ; 
		beSticky ;
		layoutInset: 0 ;
		dropEnabled: true! !

!DockingBarMorph methodsFor: 'initialize-release' stamp: 'mt 7/26/2016 12:04'!
setDefaultParameters
	"private - set the default parameter using Preferences as the inspiration source"

	self
		color: (self userInterfaceTheme color ifNil: [Color r: 0.9 g: 0.9 b: 0.9]);
		borderStyle: (self userInterfaceTheme borderStyle ifNil: [BorderStyle default]);
		borderColor: (self userInterfaceTheme borderColor ifNil: [Color gray]);
		borderWidth: (self userInterfaceTheme borderWidth ifNil: [0]).
		
	self extent: (Preferences standardMenuFont height asPoint).! !

!DockingBarMorph methodsFor: 'update' stamp: 'mt 7/26/2016 12:00'!
applyUserInterfaceTheme

	| colorToUse |
	gradientRamp := nil.
	
	super applyUserInterfaceTheme.
	
	self setDefaultParameters.
	
	"Update properties of separating lines."
	colorToUse := self userInterfaceTheme lineColor ifNil: [Color gray: 0.9].
	self submorphs
		select: [:ea | ea knownName = #line]
		thenDo: [:line |
			line
				color: colorToUse;
				extent: (self userInterfaceTheme lineWidth ifNil: [2]) asPoint;
				borderStyle: (self userInterfaceTheme lineStyle ifNil: [BorderStyle inset]);
				borderColor: colorToUse].! !


!DockingBarMorph class methodsFor: 'preferences' stamp: 'mt 7/10/2016 12:31'!
themeProperties

	^ super themeProperties, {
		{ #borderColor. 'Colors'. 'Color of the menu''s border.' }.
		{ #borderWidth. 'Geometry'. 'Width of the menu''s border.' }.
		{ #borderStyle. 'Borders'. 'Whether to use a plain border, inset, or outset.' }.
		{ #color. 'Colors'. 'Background color of the menu.' }.

		{ #lineColor. 'Colors'. 'Color of the separators between menu items.' }.
		{ #lineStyle. 'Colors'. 'Use border-style to change appearance.' }.
		{ #lineWidth. 'Geometry'. 'How big the separators should be.' }.
	}! !


!MenuIcons class methodsFor: 'accessing - icons' stamp: 'mt 7/10/2016 12:54'!
squeakLogoIcon
^ Icons
		at: #squeakLogoIcon ifAbsentPut: [(Form
	extent: 24@16
	depth: 32
	fromArray: #( 0 0 67108865 2063597569 1778384897 889192449 0 0 0 0 0 0 0 0 0 0 0 0 1610612737 1811939329 1610612737 0 0 0 0 0 905969665 889192449 0 1442840577 788529153 0 0 0 0 0 0 0 0 0 0 1711276033 469762049 0 1728053249 0 0 0 0 0 1006632961 704643073 0 0 1895825409 234881025 0 0 0 0 0 0 0 0 1207959553 838860801 0 0 1728053249 0 0 0 0 0 738197505 989855745 0 0 167772161 1862270977 0 0 0 0 0 0 0 83886081 1879048193 0 0 0 1744830465 0 0 0 0 0 469762049 1258291201 0 0 0 1543503873 520093697 0 0 0 0 0 0 1543503873 335544321 0 0 0 1761607681 0 0 0 0 0 16777217 1644167169 0 0 0 33554433 67108865 0 0 0 0 0 0 33554433 0 0 0 0 1795162113 0 0 0 0 0 0 1711276033 0 0 0 0 0 117440513 0 0 0 0 0 0 0 0 0 100663297 1677721601 0 0 0 0 0 0 1744830465 0 0 0 0 3019898881 4009754625 1056964609 0 0 1090519041 3204448257 1325400065 0 0 0 889192449 889192449 0 0 0 0 0 0 1392508929 352321537 0 0 637534209 4278190081 4278190081 2650800129 0 0 3372220417 4278190081 3758096385 0 0 0 1560281089 117440513 0 0 0 0 0 0 0 0 0 0 134217729 4060086273 4278190081 1929379841 0 0 3187671041 4278190081 3590324225 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 704643073 1577058305 83886081 0 0 771751937 2667577345 956301313 0 0 0 0 0 0 0 0 0 0 369098753 973078529 1577058305 1728053249 1711276033 1325400065 805306369 218103809 0 855638017 922746881 16777217 0 268435457 788529153 1308622849 1593835521 1056964609 436207617 16777217 0 0 1442840577 1761607681 1409286145 788529153 301989889 872415233 1124073473 1627389953 1929379841 973078529 2030043137 4278190081 4278190081 2164260865 1040187393 1946157057 1577058305 939524097 452984833 687865857 1325400065 1778384897 1795162113 1392508929 0 0 855638017 1795162113 1677721601 855638017 620756993 436207617 486539265 285212673 2030043137 4278190081 4278190081 2164260865 486539265 1023410177 922746881 1157627905 1392508929 1845493761 1744830465 486539265 0 0 201326593 2013265921 1006632961 0 553648129 1694498817 1778384897 1694498817 1728053249 738197505 0 855638017 922746881 16777217 738197505 1728053249 1744830465 1778384897 1476395009 251658241 117440513 1476395009 1879048193 150994945 0 16777217 0 486539265 1426063361 16777217 0 0 0 0 0 0 0 0 0 0 0 0 234881025 1728053249 486539265 0 167772161 50331649)
	offset: 0@0)]! !

!MenuIcons class methodsFor: 'accessing - icons' stamp: 'mt 7/10/2016 12:57'!
squeakLogoIconColorized: aColor

	^ self squeakLogoIcon
		collectColors: [:c | aColor alpha: c alpha]! !


!MenuItemMorph methodsFor: 'accessing' stamp: 'mt 7/9/2016 15:59'!
colorToUse

	^ isSelected & isEnabled
		ifTrue: [ self userInterfaceTheme selectionTextColor ifNil: [Color white] ]
		ifFalse: [ color ]! !

!MenuItemMorph methodsFor: 'drawing' stamp: 'mt 7/9/2016 15:19'!
drawBackgroundOn: aCanvas 

	isSelected & isEnabled
		ifTrue: [aCanvas fillRectangle: self bounds fillStyle: self selectionFillStyle].! !

!MenuItemMorph methodsFor: 'drawing' stamp: 'mt 7/9/2016 15:16'!
drawIconOn: aCanvas 

	self hasIcon ifTrue: [
		| iconForm | 
		iconForm := self iconForm.
		aCanvas
			translucentImage: iconForm 
			at: self bounds left @ (self top + (self height - iconForm height // 2)).].! !

!MenuItemMorph methodsFor: 'drawing' stamp: 'mt 7/9/2016 15:42'!
drawLabelOn: aCanvas 

	| stringBounds |	
	stringBounds := bounds.
	
	self hasIcon ifTrue: [
		stringBounds := stringBounds left: stringBounds left + self iconForm width + 2 ].
	self hasMarker ifTrue: [
		stringBounds := stringBounds left: stringBounds left + self submorphBounds width + 8 ].
	
	stringBounds := stringBounds top: stringBounds top + stringBounds bottom - self fontToUse height // 2.

	aCanvas
		drawString: contents
		in: stringBounds
		font: self fontToUse
		color: self colorToUse.! !

!MenuItemMorph methodsFor: 'drawing' stamp: 'mt 7/9/2016 15:20'!
drawOn: aCanvas 

	self
		drawBackgroundOn: aCanvas;
		drawIconOn: aCanvas;
		drawLabelOn: aCanvas;
		drawSubMenuMarkerOn: aCanvas.! !

!MenuItemMorph methodsFor: 'drawing' stamp: 'mt 7/9/2016 15:19'!
drawSubMenuMarkerOn: aCanvas 

	self hasSubMenu ifTrue: [
		| subMenuMarker subMenuMarkerPosition | 
		subMenuMarker := self subMenuMarker.
		subMenuMarkerPosition := self right - subMenuMarker width @ (self top + self bottom - subMenuMarker height // 2).
		aCanvas paintImage: subMenuMarker at: subMenuMarkerPosition ]! !

!MenuItemMorph methodsFor: 'updating' stamp: 'mt 7/9/2016 15:58'!
applyUserInterfaceTheme

	super applyUserInterfaceTheme.
	
	self
		color: (self userInterfaceTheme textColor ifNil: [Color black]);
		font: (self userInterfaceTheme font ifNil: [TextStyle defaultFont]).
	! !

!MenuItemMorph methodsFor: 'initialization' stamp: 'mt 7/9/2016 15:57'!
initialize
	"initialize the state of the receiver"
	super initialize.
	""
	
	contents := ''.
	hasFocus := false.
	isEnabled := true.
	subMenu := nil.
	isSelected := false.
	target := nil.
	selector := nil.
	arguments := nil.
	font := self userInterfaceTheme font ifNil: [TextStyle defaultFont].
	self
		color: (self userInterfaceTheme textColor ifNil: [Color black]);
		hResizing: #spaceFill;
		vResizing: #shrinkWrap! !

!MenuItemMorph methodsFor: 'layout' stamp: 'mt 7/9/2016 15:31'!
minWidth

	| subMenuWidth iconWidth markerWidth |
	subMenuWidth := self hasSubMenu
		ifTrue: [ 10 ]
		ifFalse: [ 0 ].
	iconWidth := self hasIcon
		ifTrue: [ self icon width + 2 ]
		ifFalse: [ 0 ].
	markerWidth := self hasMarker
		ifTrue: [ self submorphBounds width + 8 ]
		ifFalse: [ 0 ].
	^(self fontToUse widthOfString: contents)
		+ subMenuWidth + iconWidth + markerWidth + self stringMargin! !

!MenuItemMorph methodsFor: 'layout' stamp: 'mt 7/9/2016 15:26'!
stringMargin

	^Preferences tinyDisplay
		ifTrue: [ 1 ]
		ifFalse: [ 6 ]! !

!MenuItemMorph methodsFor: 'private' stamp: 'dgd 9/1/2004 18:07'!
rightArrow

	^ SubMenuMarker! !

!MenuItemMorph methodsFor: 'private' stamp: 'mt 7/22/2016 11:58'!
selectionFillStyle
	" Answer the fill style to use with the receiver is the selected element "

	| fill baseColor |
	baseColor := self userInterfaceTheme selectionColor ifNil: [Color r: 0.4 g: 0.5 b: 0.7].
	MenuMorph gradientMenu ifFalse: [ ^baseColor ].

	fill := GradientFillStyle ramp: { 
		0.0 -> baseColor twiceLighter. 
		1 -> baseColor twiceDarker }.
	fill origin: self topLeft.
	fill direction: 0 @ self height.
	^ fill! !

!MenuItemMorph methodsFor: 'private' stamp: 'kb 11/19/2009 17:59'!
subMenuMarker
	"private - answer the form to be used as submenu marker"
	
	^self rightArrow! !


!DockingBarItemMorph methodsFor: 'selecting' stamp: 'mt 7/26/2016 13:10'!
adjacentTo

	| roundedCornersOffset verticalOffset |
	roundedCornersOffset := MenuMorph roundedMenuCorners
		ifTrue: [Morph preferredCornerRadius negated]
		ifFalse: [0].
	verticalOffset := 2.

	owner isFloating
		ifTrue: [^ {self bounds bottomLeft + (roundedCornersOffset @ verticalOffset)}].
	owner isAdheringToTop
		ifTrue: [^ {self bounds bottomLeft + (roundedCornersOffset @ verticalOffset)}].
	owner isAdheringToLeft
		ifTrue: [^ {self bounds topRight + (roundedCornersOffset @ verticalOffset)}].
	owner isAdheringToBottom
		ifTrue: [^ {self bounds topLeft + (roundedCornersOffset @ verticalOffset)}].
	owner isAdheringToRight
		ifTrue: [^ {self bounds topLeft + (roundedCornersOffset @ verticalOffset negated)}].
	^ {self bounds bottomLeft + (roundedCornersOffset @ 5)}! !

!DockingBarItemMorph methodsFor: 'drawing' stamp: 'mt 7/9/2016 15:39'!
drawIconOn: aCanvas 

	| pos |
	self hasIcon ifTrue: [
		| iconForm | 
		iconForm := self iconForm.

		pos := (contents
			ifEmpty: [self left + (self width - iconForm width // 2)]
			ifNotEmpty: [self left])
				@ (self top + (self height - iconForm height // 2)).

		aCanvas
			translucentImage: iconForm 
			at: pos].! !

!DockingBarItemMorph methodsFor: 'drawing' stamp: 'mt 7/9/2016 15:42'!
drawLabelOn: aCanvas 

	| stringBounds |	
	self contents ifEmpty: [^ self].
	
	stringBounds := bounds.
	
	self hasIcon ifTrue: [
		stringBounds := stringBounds left: stringBounds left + self iconForm width + 2 ].
	
	"Vertical centering."
	stringBounds := stringBounds top: stringBounds top + stringBounds bottom - self fontToUse height // 2.
	"Horizontal centering."
	stringBounds := stringBounds left: stringBounds left + (stringBounds width - (self fontToUse widthOfString: contents) // 2) abs.

	aCanvas
		drawString: contents
		in: stringBounds
		font: self fontToUse
		color: self colorToUse.! !

!DockingBarItemMorph methodsFor: 'drawing' stamp: 'mt 7/9/2016 15:21'!
drawSubMenuMarkerOn: aCanvas 
	"Ignore."! !


!MenuItemMorph class methodsFor: 'defaults' stamp: 'mt 7/10/2016 11:49'!
defaultSubMenuMarker

	| f |
	f := Form
		extent: 5@9
		fromArray: #(2147483648 3221225472 3758096384 4026531840 4160749568 4026531840 3758096384 3221225472 2147483648)
		offset: 0@0.
	^ ColorForm mappingWhiteToTransparentFrom: f! !

!MenuItemMorph class methodsFor: 'preferences' stamp: 'mt 7/10/2016 11:49'!
applyUserInterfaceTheme

	SubMenuMarker := (UserInterfaceTheme current get: #subMenuMarker for: self)
		ifNil: [self defaultSubMenuMarker].! !

!MenuItemMorph class methodsFor: 'preferences' stamp: 'mt 7/10/2016 12:31'!
themeProperties

	^ super themeProperties, {	
		{ #font. 'Fonts'. 'Font for menu items.' }.
		{ #textColor. 'Colors'. 'Color for the menu item''s labels.' }.
		{ #selectionColor. 'Colors'. 'Color used for items when hovering or selecting them.' }.
		{ #selectionTextColor. 'Colors'. 'Color used for label when hovering or selecting them.' }.
		{ #subMenuMarker. 'Forms'. 'The form to be used to indicate a submenu.' }.
	}! !

!MenuItemMorph class methodsFor: 'class initialization' stamp: 'mt 7/10/2016 11:49'!
initialize
	"MenuItemMorph initialize"

	SubMenuMarker := self defaultSubMenuMarker.
! !


!MenuMorph methodsFor: 'construction' stamp: 'mt 7/9/2016 14:37'!
addTitle: aString icon: aForm updatingSelector: aSelector updateTarget: aTarget 
	"Add a title line at the top of this menu Make aString its initial contents. If aSelector is not nil, then periodically obtain fresh values for its contents by sending aSelector to aTarget."
	
	| container fontToUse colorToUse labelArea |
	
	container := AlignmentMorph newRow.
	self setTitleParametersFor: container.
	
	aForm ifNotNil: [:form | container addMorphBack: form asMorph].

	fontToUse := self userInterfaceTheme titleFont ifNil: [TextStyle defaultFont].
	colorToUse := self userInterfaceTheme titleTextColor ifNil: [Color black].
	
	"Build the label."
	aSelector
		ifNil: [
			labelArea := AlignmentMorph newColumn
				color: Color transparent;
				vResizing: #shrinkWrap;
				wrapCentering: #center;
				cellPositioning: #topCenter.
			aString asString linesDo: [:line | labelArea 
					addMorphBack: ((StringMorph contents: line 
										font: fontToUse)
										color: colorToUse;
										yourself)]]
		ifNotNil: [
			labelArea := UpdatingStringMorph on: aTarget selector: aSelector.
			labelArea font: fontToUse.
			labelArea color: colorToUse.
			labelArea useStringFormat.
			labelArea lock].
		
	container addMorphBack: labelArea.
	""
	container setProperty: #titleString toValue: aString.
	self addMorphFront: container.
	""
	(self hasProperty: #needsTitlebarWidgets)
		ifTrue: [self addStayUpIcons]! !

!MenuMorph methodsFor: 'construction' stamp: 'mt 7/9/2016 13:06'!
addLine
	"Append a divider line to this menu. Suppress duplicate lines."

	| colorToUse |
	self hasItems ifFalse: [^ self].
	self lastSubmorph knownName = #line ifTrue: [^ self].
	
	colorToUse := self userInterfaceTheme lineColor ifNil: [Color gray: 0.9].
	self addMorphBack: (Morph new
		color: colorToUse;
		hResizing: #spaceFill;
		height: (self userInterfaceTheme lineWidth ifNil: [2]);
		borderStyle: (self userInterfaceTheme lineStyle ifNil: [BorderStyle inset]);
		borderColor: colorToUse;
		borderWidth: 1;
		name: #line; "see above"
		yourself).! !

!MenuMorph methodsFor: 'construction' stamp: 'mt 7/9/2016 14:21'!
addStayUpIcons
	| title closeBox pinBox |
	title := submorphs
				detect: [:ea | ea hasProperty: #titleString]
				ifNone: [
					"Called too soon. Will add stay-up icons when title is added."
					self setProperty: #needsTitlebarWidgets toValue: true.
					^ self].
	closeBox := SystemWindowButton new target: self;
				 actionSelector: #delete;
				 labelGraphic: self class closeBoxImage;
				 color: Color transparent;
				 extent: self class closeBoxImage extent;
				 borderWidth: 0.
	pinBox := SystemWindowButton new target: self;
				 actionSelector: #stayUp:;
				 arguments: {true};
				 labelGraphic: self class pushPinImage;
				 color: Color transparent;
				 extent: self class pushPinImage extent;
				 borderWidth: 0.
	Preferences noviceMode ifTrue: [
		closeBox setBalloonText: 'close this menu'.
		pinBox setBalloonText: 'keep this menu up'].
		
	title
		addMorphFront: closeBox;
		addMorphBack: pinBox.
		
	self setProperty: #hasTitlebarWidgets toValue: true.
	self removeProperty: #needsTitlebarWidgets.
	self removeStayUpItems! !

!MenuMorph methodsFor: 'control' stamp: 'mt 7/26/2016 11:49'!
updateColor
	| fill title cc |
	self class gradientMenu
		ifFalse: [^ self].
	(self fillStyle == self color) not 
		ifTrue: [^ self]. "Don't apply the gradient more than once"
	""
	title := self allMorphs
				detect: [:each | each hasProperty: #titleString]
				ifNone: [].
	
	cc := self color adjustSaturation: -0.08 brightness: 0.4.
	fill := GradientFillStyle ramp: {
		0.0 -> (title ifNil: [cc] ifNotNil: [cc muchLighter]). 
		0.25 -> (self color mixed: 0.5 with: cc). 
		1.0 -> self color}.
	""
	fill
		radial: false;
		origin: self topLeft;
		direction: 0 @ self height.
	""
	" 
	update the title color"

	title ifNotNil: [fill direction: 0@ title height].
	self fillStyle: fill.
	title ifNil: [^ self].
	""
	fill := GradientFillStyle ramp: {
			0.0 -> title color twiceLighter.
			1 -> title color twiceDarker}.
	""
	fill
		origin: title topLeft;
		direction: title width @ 0.
	""
	title fillStyle: fill! !

!MenuMorph methodsFor: 'initialization' stamp: 'mt 7/9/2016 13:38'!
initialize
	super initialize.

	bounds := 0 @ 0 corner: 40 @ 10.

	self setDefaultParameters.

	self changeTableLayout.
	self listDirection: #topToBottom.
	self hResizing: #shrinkWrap.
	self vResizing: #shrinkWrap.
	defaultTarget := nil.
	selectedItem := nil.
	stayUp := false.
	popUpOwner := nil.! !

!MenuMorph methodsFor: 'initialization' stamp: 'mt 7/9/2016 13:40'!
setDefaultParameters
	"change the receiver's appareance parameters"

	self
		color: (self userInterfaceTheme color ifNil: [Color r: 0.9 g: 0.9 b: 0.9]);
		borderStyle: (self userInterfaceTheme borderStyle ifNil: [BorderStyle default]);
		borderColor: (self userInterfaceTheme borderColor ifNil: [Color gray]);
		borderWidth: (self userInterfaceTheme borderWidth ifNil: [1]).

	Preferences menuAppearance3d ifTrue: [self addDropShadow].
	
	self layoutInset: 3.
! !

!MenuMorph methodsFor: 'initialization' stamp: 'mt 7/25/2016 14:08'!
setTitleParameters
		
	 self setTitleParametersFor: (self allMorphs
			detect: [:each | each hasProperty: #titleString]
			ifNone: [^ self]).! !

!MenuMorph methodsFor: 'initialization' stamp: 'mt 7/9/2016 14:44'!
setTitleParametersFor: aMenuTitle 

	aMenuTitle
		color: (self userInterfaceTheme titleColor ifNil: [Color transparent]);
		borderStyle: (self userInterfaceTheme titleBorderStyle ifNil: [BorderStyle default]);
		borderColor: (self userInterfaceTheme titleBorderColor ifNil: [Color r: 0.6 g: 0.7 b: 1]);
		borderWidth: (self userInterfaceTheme titleBorderWidth ifNil: [0]);
		cornerStyle: (self wantsRoundedCorners ifTrue: [#rounded] ifFalse: [#square]);
		vResizing: #shrinkWrap;
		wrapCentering: #center;
		cellPositioning: #center;
		cellInset: 5;
		layoutInset: (5@0 corner: 5@0).! !

!MenuMorph methodsFor: 'update' stamp: 'mt 7/9/2016 15:01'!
applyUserInterfaceTheme

	| colorToUse |
	super applyUserInterfaceTheme.
	
	self setDefaultParameters.
	
	"Update properties of separating lines."
	colorToUse := self userInterfaceTheme lineColor ifNil: [Color gray: 0.9].
	self submorphs
		select: [:ea | ea knownName = #line]
		thenDo: [:line |
			line
				color: colorToUse;
				height: (self userInterfaceTheme lineWidth ifNil: [2]);
				borderStyle: (self userInterfaceTheme lineStyle ifNil: [BorderStyle inset]);
				borderColor: colorToUse].! !


!MenuMorph class methodsFor: '*51Deprecated' stamp: 'mt 7/10/2016 13:27'!
menuBorderColor

	self deprecated: 'mt: Use UI themes.'.
	^ (UserInterfaceTheme current get: #borderColor for: self) ifNil: [(Color r: 0.2 g: 0.3 b: 0.9)]! !

!MenuMorph class methodsFor: '*51Deprecated' stamp: 'mt 7/10/2016 13:27'!
menuBorderWidth
	
	self deprecated: 'mt: Use UI themes.'.
	^ (UserInterfaceTheme current get: #borderWidth for: self) ifNil: [2]! !

!MenuMorph class methodsFor: '*51Deprecated' stamp: 'mt 7/10/2016 13:28'!
menuColor
	
	self deprecated: 'mt: Use UI themes.'.
	^ (UserInterfaceTheme current get: #color for: self) ifNil: [(Color r: 0.9 g: 0.9 b: 0.9)]! !

!MenuMorph class methodsFor: '*51Deprecated' stamp: 'mt 7/10/2016 13:28'!
menuLineColor
	
	self deprecated: 'mt: Use UI themes.'.
	^ (UserInterfaceTheme current get: #lineColor for: self) ifNil: [(Color r: 0.6 g: 0.7 b: 1)]! !

!MenuMorph class methodsFor: '*51Deprecated' stamp: 'mt 7/10/2016 13:29'!
menuSelectionColor
	
	self deprecated: 'mt: Use UI themes.'.
	^ (UserInterfaceTheme current get: #selectionColor for: MenuItemMorph) ifNil: [(Color r: 0.4 g: 0.5 b: 0.7)]! !

!MenuMorph class methodsFor: '*51Deprecated' stamp: 'mt 7/10/2016 13:29'!
menuTitleBorderColor
	
	self deprecated: 'mt: Use UI themes.'.
	^ (UserInterfaceTheme current get: #titleBorderColor for: self) ifNil: [(Color r: 0.6 g: 0.7 b: 1)]! !

!MenuMorph class methodsFor: '*51Deprecated' stamp: 'mt 7/10/2016 13:29'!
menuTitleBorderWidth

	self deprecated: 'mt: Use UI themes.'.
	^ (UserInterfaceTheme current get: #titleBorderWidth for: self) ifNil: [0]! !

!MenuMorph class methodsFor: '*51Deprecated' stamp: 'mt 7/10/2016 13:30'!
menuTitleColor
	
	self deprecated: 'mt: Use UI themes.'.
	^ (UserInterfaceTheme current get: #titleColor for: self) ifNil: [ Color transparent]! !

!MenuMorph class methodsFor: 'preferences' stamp: 'mt 7/10/2016 12:31'!
themeProperties

	^ super themeProperties, {
		{ #borderColor. 'Colors'. 'Color of the menu''s border.' }.
		{ #borderWidth. 'Borders'. 'Width of the menu''s border.' }.
		{ #borderStyle. 'Borders'. 'Whether to use a plain border, inset, or outset.' }.
		{ #color. 'Colors'. 'Background color of the menu.' }.

		{ #titleBorderColor. 'Colors'. 'Color of the menu title border.' }.
		{ #titleBorderWidth. 'Geometry'. 'Width of the menu title border.' }.
		{ #titleBorderStyle. 'Borders'. 'Whether to use a plain border, inset, or outset for the title.' }.
		{ #titleColor. 'Colors'. 'Background color of the menu'' title.' }.
		{ #titleFont. 'Fonts'. 'Font for menu title.' }.
		{ #titleTextColor. 'Colors'. 'Color for the menu title label.' }.

		{ #lineColor. 'Colors'. 'Color of the separators between menu items.' }.
		{ #lineStyle. 'Colors'. 'Use border-style to change appearance.' }.
		{ #lineWidth. 'Geometry'. 'How big the separators should be.' }.
	}! !


!TheWorldMainDockingBar methodsFor: 'construction' stamp: 'mt 7/26/2016 12:53'!
createDockingBar
	"Create a docking bar from the receiver's representation"
	
	| dockingBar |
	dockingBar := DockingBarMorph new
		adhereToTop;
		color: MenuMorph menuColor;
		autoGradient: MenuMorph gradientMenu;
		borderWidth: 0.
	self fillDockingBar: dockingBar.
	self labelIfNeeded: dockingBar.
	^ dockingBar! !

!TheWorldMainDockingBar methodsFor: 'construction' stamp: 'mt 7/10/2016 13:03'!
squeakMenuOn: aDockingBar 
	"Private - fill the given docking bar"
	
	aDockingBar addItem: [ :item |
		item
			contents: '';
			icon: (MenuIcons squeakLogoIconColorized:
				(self userInterfaceTheme logoColor ifNil: [Color black]));
			selectedIcon: (MenuIcons squeakLogoIconColorized:
				(self userInterfaceTheme selectionLogoColor ifNil: [Color white]));
			addSubMenu: [ :menu | 
				self
					aboutMenuItemOn: menu;
					updateMenuItemOn: menu.
				menu addLine.
				self 
					saveMenuItemOn: menu;
					saveAsMenuItemOn: menu;
					saveAsNewVersionMenuItemOn: menu.
				menu addLine.
				self
					saveAndQuitMenuItemOn: menu;
					quitMenuItemOn: menu ] ]! !

!TheWorldMainDockingBar methodsFor: 'private' stamp: 'mt 7/10/2016 12:44'!
colorToUse

	^ self userInterfaceTheme textColor ifNil: [Color black]! !

!TheWorldMainDockingBar methodsFor: 'private' stamp: 'mt 7/10/2016 12:44'!
fontToUse

	^ self userInterfaceTheme font ifNil: [TextStyle defaultFont]! !

!TheWorldMainDockingBar methodsFor: 'right side' stamp: 'mt 7/10/2016 12:44'!
clockOn: aDockingBar 
	aDockingBar 
		addMorphBack: (ClockMorph new
			font: self fontToUse;
			color: self colorToUse;
			show24hr: self class twentyFourHourClock;
			showSeconds: self class showSecondsInClock;
			yourself);
		addDefaultSpace! !

!TheWorldMainDockingBar methodsFor: 'updating' stamp: 'mt 7/10/2016 12:33'!
applyUserInterfaceTheme
	"Do it on the class-side."! !


!TheWorldMainDockingBar class methodsFor: 'preferences' stamp: 'mt 7/10/2016 12:33'!
applyUserInterfaceTheme

	self updateInstances.! !

!TheWorldMainDockingBar class methodsFor: 'preferences' stamp: 'mt 7/10/2016 13:01'!
themeProperties

	^ {
		{ #textColor. 'Colors'. 'The color for the clock and other labels.' }.
		{ #font. 'Fonts'. 'The font for the clock and other labels.' }.
		{ #logoColor. 'Colors'. 'The color of the Squeak logo.' }.
		{ #selectionLogoColor. 'Colors'. 'The color of the Squeak logo when it is selected.' }.
	}! !

MenuMorph class removeSelector: #menuBorderColor:!
MenuMorph class removeSelector: #menuBorderWidth:!
MenuMorph class removeSelector: #menuColor:!
MenuMorph class removeSelector: #menuLineColor:!
MenuMorph class removeSelector: #menuSelectionColor:!
MenuMorph class removeSelector: #menuTitleBorderColor:!
MenuMorph class removeSelector: #menuTitleBorderWidth:!
MenuMorph class removeSelector: #menuTitleColor:!

!MenuMorph class reorganize!
('example' example)
('images' closeBoxImage closeBoxImageFlat closeBoxImageGradient pushPinImage)
('instance creation' entitled: fromArray: initialize)
('*51Deprecated' menuBorderColor menuBorderWidth menuColor menuLineColor menuSelectionColor menuTitleBorderColor menuTitleBorderWidth menuTitleColor)
('utilities' chooseFrom:lines:title: chooseFrom:values:lines:title: confirm: confirm:orCancel: confirm:trueChoice:falseChoice: inform:)
('preferences' gradientMenu gradientMenu: roundedMenuCorners roundedMenuCorners: themeProperties)
!

Morph subclass: #MenuMorph
	instanceVariableNames: 'defaultTarget selectedItem stayUp popUpOwner activeSubMenu'
	classVariableNames: 'CloseBoxImage CloseBoxImageFlat CloseBoxImageGradient GradientMenu MenuBorderColor MenuBorderWidth MenuColor MenuLineColor MenuSelectionColor MenuTitleBorderColor MenuTitleBorderWidth MenuTitleColor PushPinImage RoundedMenuCorners'
	poolDictionaries: ''
	category: 'Morphic-Menus'!
MenuItemMorph initialize!

!MenuItemMorph class reorganize!
('defaults' defaultSubMenuMarker)
('preferences' applyUserInterfaceTheme themeProperties)
('class initialization' initialize)
('*eToys-scripting' additionsToViewerCategories)
!

DockingBarItemMorph removeSelector: #drawOn:!
DockingBarItemMorph removeSelector: #minWidth!
DockingBarItemMorph removeSelector: #stringMargin!
DockingBarItemMorph removeSelector: #subMenuMarker!

!MenuItemMorph reorganize!
('accessing' action: adaptToWorld: addSubMenu: addUpdatingSubMenu: allWordingsNotInSubMenus: arguments arguments: colorToUse contentString contentString: contents: contents:withMarkers: contents:withMarkers:inverse: hasIcon hasIconOrMarker hasMarker hasSubMenu hasSubMenu: help: icon icon: isEnabled isEnabled: isStayUpItem itemWithWording: selector selector: subMenu subMenu: subMenuUpdater:selector: subMenuUpdater:selector:arguments: target target:)
('copying' veryDeepFixupWith: veryDeepInner:)
('drawing' drawBackgroundOn: drawIconOn: drawLabelOn: drawOn: drawSubMenuMarkerOn:)
('events' activateOwnerMenu: activateSubmenu: doButtonAction handleMouseUp: handlesMouseDown: handlesMouseOver: handlesMouseOverDragging: invokeWithEvent: mouseDown: mouseEnter: mouseEnterDragging: mouseUp:)
('grabbing' aboutToBeGrabbedBy: duplicateMorph:)
('updating' applyUserInterfaceTheme)
('initialization' defaultBounds deleteIfPopUp: initialize)
('layout' minHeight minWidth stringMargin)
('layout-properties' hResizing vResizing)
('meta actions' wantsHaloFromClick)
('selecting' adjacentTo deselect: isSelected isSelected: select:)
('private' bottomArrow createSubmenu createUpdatingSubmenu deselectItem iconForm leftArrow notObsolete offImage onImage rightArrow selectionFillStyle subMenuMarker upArrow updateLayoutInDockingBar)
('submorphs-accessing' noteNewOwner:)
('change reporting' ownerChanged)
('browse' browseAllImplementorsOfRealSelector buildDebugMenu: realSelector)
('geometry-testing' triangle:containsPoint:)
('testing' isMenuItemMorph)
!

MenuIcons class removeSelector: #squeakLogoInvertedIcon!
Morph subclass: #DockingBarMorph
	instanceVariableNames: 'originalColor gradientRamp fillsOwner avoidVisibleBordersAtEdge autoGradient selectedItem activeSubMenu oldKeyboardFocus oldMouseFocus'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Morphic-Menus-DockingBar'!
"Postscript:
Leave the line above, and replace the rest of this comment by a useful one.
Executable statements should follow this comment, and should
be separated by periods, with no exclamation points (!!).
Be sure to put any further comments in double-quotes, like this one."

MenuIcons initializeIcons.
TheWorldMainDockingBar updateInstances.
!

